<?php
namespace IchHabRecht\Devtools\Slot\Extensionmanager;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Nicole Cordes <typo3@cordes.co>, CPS-IT GmbH
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use IchHabRecht\Devtools\Controller\Slot\AbstractSlotController;
use IchHabRecht\Devtools\Utility\ExtensionUtility;
use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Package\Exception\UnknownPackageException;
use TYPO3\CMS\Core\Package\PackageManager;
use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Adds icons to extension manager list view
 *
 * @author Nicole Cordes <typo3@cordes.co>
 */
class ProcessActions
{
    /**
     * @var ExtensionUtility
     */
    private $extensionUtility;

    /**
     * @var IconFactory
     */
    protected $iconFactory;

    /**
     * @var bool
     */
    protected $isJavascriptIncluded = false;

    /**
     * @var UriBuilder
     */
    private $uriBuilder;

    public function __construct(ExtensionUtility $extensionUtility, IconFactory $iconFactory, UriBuilder $uriBuilder)
    {
        $this->extensionUtility = $extensionUtility;
        $this->iconFactory = $iconFactory;
        $this->uriBuilder = $uriBuilder;
    }

    /**
     * @param array $extension
     * @param array $actions
     * @return bool
     */
    public function addActions($extension, &$actions)
    {
        if ($this->isExtensionModified($extension)) {
            if (!$this->isJavascriptIncluded) {
                $this->includeJavascript();
            }
            $actions[] = $this->markModifiedExtension($extension);
            $actions[] = $this->updateExtensionConfigurationFile($extension);
        } else {
            $actions[] = '<span class="btn btn-default disabled">'
                . $this->iconFactory->getIcon('empty-empty', Icon::SIZE_SMALL)->render() . '</span>';
            $actions[] = '<span class="btn btn-default disabled">'
                . $this->iconFactory->getIcon('empty-empty', Icon::SIZE_SMALL)->render() . '</span>';
        }

        return false;
    }

    /**
     * @param array $extension
     * @return bool
     */
    protected function isExtensionModified($extension)
    {
        if (!empty($extension['_md5_values_when_last_written'])) {
            $md5HashArray = $this->extensionUtility->getMd5HashArrayForExtension($extension['key']);

            return $extension['_md5_values_when_last_written'] !== serialize($md5HashArray);
        }

        return false;
    }

    /**
     * @param array $extension
     * @return string
     */
    protected function markModifiedExtension($extension)
    {
        $title = $GLOBALS['LANG']->sL(AbstractSlotController::LANGUAGE_FILE
            . ':slot.extensionmanager.process_actions.modified_files.title');

        return '<a href="'
            . $this->uriBuilder->buildUriFromRoute('ajax_DevtoolsFilesModifiedList', ['extensionKey' => $extension['key']])
            . '" class="btn btn-default list-modified-files" title="' . htmlspecialchars($title) . '">'
            . $this->iconFactory->getIcon('status-dialog-warning', Icon::SIZE_SMALL)->render() . '</a>';
    }

    /**
     * @param array $extension
     * @return string
     */
    protected function updateExtensionConfigurationFile($extension)
    {
        try {
            $packageManager = GeneralUtility::makeInstance(PackageManager::class);
            $package = $packageManager->getPackage($extension['key']);
            if (!$package->isProtected() && $package->getPackageMetaData()->getPackageType() !== 'typo3-cms-framework') {
                $configurationFile = $package->getPackagePath() . 'ext_emconf.php';
                if (is_writable($configurationFile)) {
                    $title = $GLOBALS['LANG']->sL(AbstractSlotController::LANGUAGE_FILE
                        . ':slot.extensionmanager.process_actions.update_configuration.title');

                    return '<a href="'
                        . $this->uriBuilder->buildUriFromRoute('ajax_DevtoolsFilesModifiedUpdate', ['extensionKey' => $extension['key']])
                        . '" class="btn btn-default update-configuration-file" title="' . $title . '">'
                        . $this->iconFactory->getIcon('actions-document-export-t3d', Icon::SIZE_SMALL)->render() . '</a>';
                }
            }
        } catch (UnknownPackageException $e) {
            return '';
        }

        return '';
    }

    /**
     * Add Javascript files and settings
     */
    protected function includeJavascript()
    {
        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
        $pageRenderer->loadRequireJsModule('TYPO3/CMS/Devtools/Devtools');
        $pageRenderer->addInlineLanguageLabel(
            'devtools.error.title',
            $GLOBALS['LANG']->sL(AbstractSlotController::LANGUAGE_FILE
                . ':slot.error.title')
        );
        $pageRenderer->addInlineLanguageLabel(
            'devtools.error.message',
            $GLOBALS['LANG']->sL(AbstractSlotController::LANGUAGE_FILE
                . ':slot.error.message')
        );
        $this->isJavascriptIncluded = true;
    }
}
